import { Yargs } from "./Contracts";
import Input from "./io/Input";
import Output from "./io/Output";
export declare type CommandConstructor = {
    new (...args: any[]): Command;
};
/**
 * Command class
 */
export default abstract class Command {
    /**
     * The command signature
     */
    protected command: string;
    /**
     * Description
     */
    protected describe: string;
    /**
     * Aliases
     */
    protected aliases: string[];
    /**
     * List of the command middlewares
     */
    protected middlewares: Function[];
    /**
     * List of the command options / arguments
     */
    protected options: any;
    /**
     * The input service of the command. We can use it to interact
     * with the user
     *
     */
    protected input: Input;
    /**
     * The output service of the command. We can use it
     * to showing feedback to the user.
     */
    protected output: Output;
    /**
     * List of Sub Commands
     */
    protected subCommands: CommandConstructor[];
    /**
     * Interact with the user just before the
     * command started.
     *
     * This is a good place to perform interaction
     * and getting the user answer, after that,
     * the user's response will also be the command
     * argument/option
     *
     * @param argv
     * @param args
     */
    protected interact(argv: any, ...args: any[]): Promise<{}>;
    /**
     * Builder to make the Sub Command for Yargs
     *
     * @param yargs
     */
    protected builder(yargs: Yargs): void;
    /**
     * Convert this command into Yargs command.
     */
    toYargs(): {
        command: any;
        aliases: string[];
        describe: string;
        middlewares: Function[];
        handler(argv: any): void;
        builder(yargs: any): void;
    };
    /**
     * Execute the command.
     *
     * @param argv
     * @param args
     */
    protected abstract execute(argv: any, ...args: any): Promise<void>;
}
