"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@fusion.io/core");
const chalk_1 = __importDefault(require("chalk"));
const Input_1 = __importDefault(require("./io/Input"));
const Output_1 = __importDefault(require("./io/Output"));
const ConsoleKernel_1 = __importDefault(require("./ConsoleKernel"));
const kebabCase = require('lodash.kebabcase');
/**
 * Command class
 */
class Command {
    constructor() {
        /**
         * The command signature
         */
        this.command = '';
        /**
         * Description
         */
        this.describe = '';
        /**
         * Aliases
         */
        this.aliases = [];
        /**
         * List of the command middlewares
         */
        this.middlewares = [];
        /**
         * List of the command options / arguments
         */
        this.options = [];
        /**
         * The input service of the command. We can use it to interact
         * with the user
         *
         */
        this.input = core_1.tokamak.make(Input_1.default);
        /**
         * The output service of the command. We can use it
         * to showing feedback to the user.
         */
        this.output = core_1.tokamak.make(Output_1.default);
        /**
         * List of Sub Commands
         */
        this.subCommands = [];
    }
    /**
     * Interact with the user just before the
     * command started.
     *
     * This is a good place to perform interaction
     * and getting the user answer, after that,
     * the user's response will also be the command
     * argument/option
     *
     * @param argv
     * @param args
     */
    interact(argv, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            return {};
        });
    }
    /**
     * Builder to make the Sub Command for Yargs
     *
     * @param yargs
     */
    builder(yargs) {
        this.subCommands.forEach(Cmd => {
            const cmd = core_1.tokamak.make(Cmd);
            yargs.command(cmd.toYargs());
        });
    }
    ;
    /**
     * Convert this command into Yargs command.
     */
    toYargs() {
        const instance = this;
        return {
            command: this.command || kebabCase(this.constructor.name),
            aliases: this.aliases,
            describe: chalk_1.default.gray(this.describe) || chalk_1.default `{gray Command created by [{cyan ${this.constructor.name}}]}`,
            middlewares: [
                (argv) => {
                    this.input.setInteractive(argv.interactive);
                    this.output.setVerbosity(argv.verbose);
                    return instance.interact(argv).then((parsed) => (Object.assign({}, argv, parsed)));
                },
                ...instance.middlewares
            ],
            handler(argv) {
                (() => __awaiter(this, void 0, void 0, function* () {
                    try {
                        yield instance.execute(argv);
                        return 0;
                    }
                    catch (error) {
                        yield instance.output.error('box', error.message);
                        yield instance.output.debug('log', error.stack);
                        core_1.tokamak.make(ConsoleKernel_1.default).emit('error', error);
                        return error.code || -1;
                    }
                }))().then(() => process.exit(0));
            },
            builder(yargs) {
                // TODO need to work with positional
                Object.entries(instance.options).forEach(([key, options]) => yargs.option(key, options));
                instance.builder(yargs);
            }
        };
    }
}
exports.default = Command;
//# sourceMappingURL=Command.js.map